#[macro_use]
extern crate rocket;

use rocket::form::Form;
use rocket::response::Redirect;
use rocket::State; // Add this line
use rocket::fs::FileServer;
use rocket::serde::json::Json;
use rocket_dyn_templates::Template;
use std::sync::Mutex;
use std::collections::HashMap;
use csv::Writer;

mod voting;

#[derive(rocket::FromForm)]
struct Vote {
    option: String,
}

type Votes = Mutex<Writer<std::fs::File>>;

#[post("/vote", data = "<vote>")]
fn vote(vote: Form<Vote>, votes: &State<Votes>) -> Redirect {
    let mut writer = votes.lock().unwrap();
    writer.write_record(&[vote.option.clone()]).unwrap();
    writer.flush().unwrap();

    Redirect::to("/")
}

#[post("/results")]
fn results() -> Json<HashMap<String, String>> {
    let election_data = voting::csv_to_election_data("votes.csv", 1, "Election".to_string());
    let blt = voting::election_data_to_blt(election_data);
    let results = voting::run_blt(blt);
    Json(results)
}

#[get("/")]
fn index() -> Template {
    let context: HashMap<&str, &str> = HashMap::new();
    Template::render("index", &context)
}

#[launch]
fn rocket() -> _ {
    let file = std::fs::OpenOptions::new()
        .create(true)
        .append(true)
        .open("votes.csv")
        .unwrap();

    let writer = csv::Writer::from_writer(file);

    rocket::build()
        .attach(Template::fairing())
        .manage(Mutex::new(writer))
        .mount("/", routes![index, vote])
        .mount("/static", FileServer::from("static"))
}